// Traveling Salesman Problem via Genetic Algorithm
// Written by David J. Stein, Esq., Fall 2004. See "License.html" for terms and conditions of use.
//
// This module contains the front-end code for the Traveling Salesman Problem.

#region Using declarations

using System;
using System.Drawing;
using System.Xml;
using System.Xml.Schema;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Threading;

#endregion

namespace GeneticAlgorithmForTravelingSalesmanProblem {

	public class TravelingSalesmanProblem : System.Windows.Forms.Form	{

		#region Variables, initialization code, and Main

		private System.Windows.Forms.PictureBox picMap;
		private System.Windows.Forms.Button btnScaleCities;
		private System.ComponentModel.Container components = null;
		private TravelingSalesmanSolution network;
		private System.Windows.Forms.Button btnInitialize;
		private System.Windows.Forms.Button btnRunGeneration;
		private System.Windows.Forms.TextBox txtStatus;
		private System.Windows.Forms.GroupBox groupBox2;
		private System.Windows.Forms.ListView lstRoutes;
		private System.Windows.Forms.Button btnSolveFull;
		private System.Timers.Timer timerMapUpdate;
		private System.Windows.Forms.Button btnClearNetwork;
		private System.Windows.Forms.Button btnSave;
		private System.Windows.Forms.Button btnLoad;
		private System.Timers.Timer timerRouteList;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.TextBox txtPopulationSize;
		private System.Windows.Forms.TextBox txtMutationRate;
		private System.Windows.Forms.TextBox txtNewEntities;
		private System.Windows.Forms.TextBox txtBuddingRate;
		private System.Windows.Forms.TextBox txtReplacementRate;

		private City movingCity = null;
		private int iMouseX = 0, iMouseY = 0;
		public Thread SolveThread = null;
		private string XmlLoadErrors = "";

		public TravelingSalesmanProblem() {
			InitializeComponent();
			network = new TravelingSalesmanSolution();
			SetTextboxes();
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(TravelingSalesmanProblem));
			this.picMap = new System.Windows.Forms.PictureBox();
			this.btnScaleCities = new System.Windows.Forms.Button();
			this.btnClearNetwork = new System.Windows.Forms.Button();
			this.btnInitialize = new System.Windows.Forms.Button();
			this.btnRunGeneration = new System.Windows.Forms.Button();
			this.txtStatus = new System.Windows.Forms.TextBox();
			this.lstRoutes = new System.Windows.Forms.ListView();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.btnSolveFull = new System.Windows.Forms.Button();
			this.timerMapUpdate = new System.Timers.Timer();
			this.btnSave = new System.Windows.Forms.Button();
			this.btnLoad = new System.Windows.Forms.Button();
			this.timerRouteList = new System.Timers.Timer();
			this.label1 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.label4 = new System.Windows.Forms.Label();
			this.label5 = new System.Windows.Forms.Label();
			this.txtPopulationSize = new System.Windows.Forms.TextBox();
			this.txtMutationRate = new System.Windows.Forms.TextBox();
			this.txtNewEntities = new System.Windows.Forms.TextBox();
			this.txtBuddingRate = new System.Windows.Forms.TextBox();
			this.txtReplacementRate = new System.Windows.Forms.TextBox();
			this.groupBox2.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.timerMapUpdate)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.timerRouteList)).BeginInit();
			this.SuspendLayout();
			// 
			// picMap
			// 
			this.picMap.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.picMap.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.picMap.Location = new System.Drawing.Point(8, 8);
			this.picMap.Name = "picMap";
			this.picMap.Size = new System.Drawing.Size(648, 217);
			this.picMap.TabIndex = 15;
			this.picMap.TabStop = false;
			this.picMap.Paint += new System.Windows.Forms.PaintEventHandler(this.picMap_Paint);
			this.picMap.MouseUp += new System.Windows.Forms.MouseEventHandler(this.picMap_MouseUp);
			this.picMap.MouseMove += new System.Windows.Forms.MouseEventHandler(this.picMap_MouseMove);
			this.picMap.MouseDown += new System.Windows.Forms.MouseEventHandler(this.picMap_MouseDown);
			// 
			// btnScaleCities
			// 
			this.btnScaleCities.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.btnScaleCities.Location = new System.Drawing.Point(120, 241);
			this.btnScaleCities.Name = "btnScaleCities";
			this.btnScaleCities.TabIndex = 17;
			this.btnScaleCities.Text = "Scale";
			this.btnScaleCities.Click += new System.EventHandler(this.btnScaleCities_Click);
			// 
			// btnClearNetwork
			// 
			this.btnClearNetwork.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.btnClearNetwork.Location = new System.Drawing.Point(32, 241);
			this.btnClearNetwork.Name = "btnClearNetwork";
			this.btnClearNetwork.TabIndex = 18;
			this.btnClearNetwork.Text = "Clear";
			this.btnClearNetwork.Click += new System.EventHandler(this.btnClearNetwork_Click);
			// 
			// btnInitialize
			// 
			this.btnInitialize.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.btnInitialize.Location = new System.Drawing.Point(208, 241);
			this.btnInitialize.Name = "btnInitialize";
			this.btnInitialize.TabIndex = 19;
			this.btnInitialize.Text = "Initialize";
			this.btnInitialize.Click += new System.EventHandler(this.btnInitialize_Click);
			// 
			// btnRunGeneration
			// 
			this.btnRunGeneration.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.btnRunGeneration.Location = new System.Drawing.Point(296, 241);
			this.btnRunGeneration.Name = "btnRunGeneration";
			this.btnRunGeneration.TabIndex = 20;
			this.btnRunGeneration.Text = "Solve (Step)";
			this.btnRunGeneration.Click += new System.EventHandler(this.btnRunGeneration_Click);
			// 
			// txtStatus
			// 
			this.txtStatus.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.txtStatus.Location = new System.Drawing.Point(8, 305);
			this.txtStatus.Name = "txtStatus";
			this.txtStatus.ReadOnly = true;
			this.txtStatus.Size = new System.Drawing.Size(648, 20);
			this.txtStatus.TabIndex = 21;
			this.txtStatus.Text = "Status: Ready.";
			// 
			// lstRoutes
			// 
			this.lstRoutes.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.lstRoutes.Location = new System.Drawing.Point(8, 16);
			this.lstRoutes.Name = "lstRoutes";
			this.lstRoutes.Size = new System.Drawing.Size(632, 256);
			this.lstRoutes.TabIndex = 22;
			this.lstRoutes.View = System.Windows.Forms.View.Details;
			// 
			// groupBox2
			// 
			this.groupBox2.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.groupBox2.Controls.Add(this.lstRoutes);
			this.groupBox2.Location = new System.Drawing.Point(8, 337);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(648, 280);
			this.groupBox2.TabIndex = 23;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "Routes";
			// 
			// btnSolveFull
			// 
			this.btnSolveFull.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.btnSolveFull.Location = new System.Drawing.Point(384, 241);
			this.btnSolveFull.Name = "btnSolveFull";
			this.btnSolveFull.TabIndex = 24;
			this.btnSolveFull.Text = "Solve (Full)";
			this.btnSolveFull.Click += new System.EventHandler(this.btnSolveFull_Click);
			// 
			// timerMapUpdate
			// 
			this.timerMapUpdate.Enabled = true;
			this.timerMapUpdate.SynchronizingObject = this;
			this.timerMapUpdate.Elapsed += new System.Timers.ElapsedEventHandler(this.timerMapUpdate_Elapsed);
			// 
			// btnSave
			// 
			this.btnSave.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.btnSave.Location = new System.Drawing.Point(472, 241);
			this.btnSave.Name = "btnSave";
			this.btnSave.TabIndex = 25;
			this.btnSave.Text = "Save";
			this.btnSave.Click += new System.EventHandler(this.btnSave_Click);
			// 
			// btnLoad
			// 
			this.btnLoad.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.btnLoad.Location = new System.Drawing.Point(560, 241);
			this.btnLoad.Name = "btnLoad";
			this.btnLoad.TabIndex = 26;
			this.btnLoad.Text = "Load";
			this.btnLoad.Click += new System.EventHandler(this.btnLoad_Click);
			// 
			// timerRouteList
			// 
			this.timerRouteList.Enabled = true;
			this.timerRouteList.Interval = 2000;
			this.timerRouteList.SynchronizingObject = this;
			this.timerRouteList.Elapsed += new System.Timers.ElapsedEventHandler(this.timerRouteList_Elapsed);
			// 
			// label1
			// 
			this.label1.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.label1.Location = new System.Drawing.Point(8, 273);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(64, 16);
			this.label1.TabIndex = 27;
			this.label1.Text = "Population";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// label2
			// 
			this.label2.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.label2.Location = new System.Drawing.Point(144, 273);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(56, 16);
			this.label2.TabIndex = 28;
			this.label2.Text = "Mutations";
			this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// label3
			// 
			this.label3.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.label3.Location = new System.Drawing.Point(264, 273);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(72, 16);
			this.label3.TabIndex = 29;
			this.label3.Text = "New Entities";
			this.label3.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// label4
			// 
			this.label4.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.label4.Location = new System.Drawing.Point(400, 273);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(48, 16);
			this.label4.TabIndex = 30;
			this.label4.Text = "Budding";
			this.label4.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// label5
			// 
			this.label5.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.label5.Location = new System.Drawing.Point(520, 273);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(80, 16);
			this.label5.TabIndex = 31;
			this.label5.Text = "Replacements";
			this.label5.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// txtPopulationSize
			// 
			this.txtPopulationSize.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.txtPopulationSize.Location = new System.Drawing.Point(80, 271);
			this.txtPopulationSize.Name = "txtPopulationSize";
			this.txtPopulationSize.Size = new System.Drawing.Size(56, 20);
			this.txtPopulationSize.TabIndex = 32;
			this.txtPopulationSize.Text = "";
			this.txtPopulationSize.TextChanged += new System.EventHandler(this.txtPopulationSize_TextChanged);
			// 
			// txtMutationRate
			// 
			this.txtMutationRate.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.txtMutationRate.Location = new System.Drawing.Point(200, 271);
			this.txtMutationRate.Name = "txtMutationRate";
			this.txtMutationRate.Size = new System.Drawing.Size(56, 20);
			this.txtMutationRate.TabIndex = 33;
			this.txtMutationRate.Text = "";
			this.txtMutationRate.TextChanged += new System.EventHandler(this.txtMutationRate_TextChanged);
			// 
			// txtNewEntities
			// 
			this.txtNewEntities.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.txtNewEntities.Location = new System.Drawing.Point(336, 271);
			this.txtNewEntities.Name = "txtNewEntities";
			this.txtNewEntities.Size = new System.Drawing.Size(56, 20);
			this.txtNewEntities.TabIndex = 34;
			this.txtNewEntities.Text = "";
			this.txtNewEntities.TextChanged += new System.EventHandler(this.txtNewEntities_TextChanged);
			// 
			// txtBuddingRate
			// 
			this.txtBuddingRate.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.txtBuddingRate.Location = new System.Drawing.Point(456, 271);
			this.txtBuddingRate.Name = "txtBuddingRate";
			this.txtBuddingRate.Size = new System.Drawing.Size(56, 20);
			this.txtBuddingRate.TabIndex = 35;
			this.txtBuddingRate.Text = "";
			this.txtBuddingRate.TextChanged += new System.EventHandler(this.txtBuddingRate_TextChanged);
			// 
			// txtReplacementRate
			// 
			this.txtReplacementRate.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.txtReplacementRate.Location = new System.Drawing.Point(600, 273);
			this.txtReplacementRate.Name = "txtReplacementRate";
			this.txtReplacementRate.Size = new System.Drawing.Size(56, 20);
			this.txtReplacementRate.TabIndex = 36;
			this.txtReplacementRate.Text = "";
			this.txtReplacementRate.TextChanged += new System.EventHandler(this.txtReplacementRate_TextChanged);
			// 
			// TravelingSalesmanProblem
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(664, 623);
			this.Controls.Add(this.txtReplacementRate);
			this.Controls.Add(this.txtBuddingRate);
			this.Controls.Add(this.txtNewEntities);
			this.Controls.Add(this.txtMutationRate);
			this.Controls.Add(this.txtPopulationSize);
			this.Controls.Add(this.txtStatus);
			this.Controls.Add(this.label5);
			this.Controls.Add(this.label4);
			this.Controls.Add(this.label3);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.btnLoad);
			this.Controls.Add(this.btnSave);
			this.Controls.Add(this.btnSolveFull);
			this.Controls.Add(this.groupBox2);
			this.Controls.Add(this.btnRunGeneration);
			this.Controls.Add(this.btnInitialize);
			this.Controls.Add(this.btnClearNetwork);
			this.Controls.Add(this.btnScaleCities);
			this.Controls.Add(this.picMap);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "TravelingSalesmanProblem";
			this.Text = "Traveling Salesman Problem via Genetic Algorithm";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.TravelingSalesmanProblem_Closing);
			this.groupBox2.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.timerMapUpdate)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.timerRouteList)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		[STAThread]
		static void Main() {
			Application.Run(new TravelingSalesmanProblem());
		}

		protected override void Dispose(bool disposing)		{
			if (disposing) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}

		#endregion

		#region Button-click event handlers

		private void btnClearNetwork_Click(object sender, System.EventArgs e) {
			network.Clear();
			SetRouteList();
			picMap.Refresh();
		}

		private void btnScaleCities_Click(object sender, System.EventArgs e) {
			// find largest coordinates
			double dMaxX = 0;
			double dMaxY = 0;
			double dMinX = picMap.Width;
			double dMinY = picMap.Height;
			foreach (object cityobject in network.cities) {
				dMaxX = Math.Max(dMaxX, ((City) cityobject).x);
				dMaxY = Math.Max(dMaxY, ((City) cityobject).y);
				dMinX = Math.Min(dMinX, ((City) cityobject).x);
				dMinY = Math.Min(dMinY, ((City) cityobject).y);
			}
			if (dMaxX == dMinX) {
				dMinX = 0;
				dMaxX = picMap.Width / 2;
			}
			else
				dMaxX = (picMap.Width - 40) / (dMaxX - dMinX);
			if (dMaxY == dMinY) {
				dMinY = 0;
				dMaxY = picMap.Height / 2;
			}
			else
				dMaxY = (picMap.Height - 40) / (dMaxY - dMinY);
			// adjust cities
			foreach (object cityobject in network.cities) {
				City city = (City) cityobject;
				city.x = 20 + (city.x - dMinX) * dMaxX;
				city.y = 20 + (city.y - dMinY) * dMaxY;
			}
			// show new coords, etc.
			network.SetDistances();
			picMap.Refresh();
		}

		private void btnInitialize_Click(object sender, System.EventArgs e) {
			network.iNumCities = network.iEntitySize = network.cities.Count;
			network.Initialize();
			SetRouteList();
			txtStatus.Text = "Status: Initialized network with " + network.lstEntities.Count + " routes.";
			txtStatus.Refresh();
		}

		private void txtPopulationSize_TextChanged(object sender, System.EventArgs e) {
			if (txtPopulationSize.Text.Length > 0) {
				try {
					network.iPopulationSize = Int32.Parse(txtPopulationSize.Text);
					network.Initialize();
				} catch (Exception ex) {}
			}
		}

		private void txtMutationRate_TextChanged(object sender, System.EventArgs e) {
			if (txtMutationRate.Text.Length > 0) {
				try {
					network.dMutationRate = Double.Parse(txtMutationRate.Text);
				} catch (Exception ex) {}
			}
		}

		private void txtNewEntities_TextChanged(object sender, System.EventArgs e) {
			if (txtNewEntities.Text.Length > 0) {
				try {
					network.dNewEntities = Double.Parse(txtNewEntities.Text);
				} catch (Exception ex) {}
			}
		}

		private void txtBuddingRate_TextChanged(object sender, System.EventArgs e) {
			if (txtBuddingRate.Text.Length > 0) {
				try {
					network.dBuddingRate = Double.Parse(txtBuddingRate.Text);
				} catch (Exception ex) {}
			}
		}

		private void txtReplacementRate_TextChanged(object sender, System.EventArgs e) {
			if (txtReplacementRate.Text.Length > 0) {
				try {
					network.dReplacementRate = Double.Parse(txtReplacementRate.Text);
				} catch (Exception ex) {}
			}
		}
		private void btnRunGeneration_Click(object sender, System.EventArgs e) {
			network.ReproducePopulation();
			Route rBestRoute = network.FindBestRoute();
			SetRouteList();
			txtStatus.Text = "Status: Ran generation. Best route = " + rBestRoute.TotalDistance(network.iDistances) + ".";
			txtStatus.Refresh();
		}

		private void btnSolveFull_Click(object sender, System.EventArgs e) {
			if (SolveThread == null) {  // start running thread
				SolveThread = new Thread(new ThreadStart(SolveFull));
				SolveThread.Start();
				btnSolveFull.Text = "Abort";
				btnSolveFull.Refresh();
			}
			else {  // abort thread
				SolveThread.Abort();
				SolveThread = null;
				btnSolveFull.Text = "Solve (Full)";
				btnSolveFull.Refresh();
			}
		}

		#endregion

		#region Other event handlers

		private void picMap_Paint(object sender, System.Windows.Forms.PaintEventArgs e) {
			Font font = new Font("Times New Roman", 7.0f);
			StringFormat format = new StringFormat();
			format.Alignment = StringAlignment.Center;
			Graphics g = e.Graphics;
			g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
			g.FillRectangle(System.Drawing.Brushes.BlanchedAlmond, e.ClipRectangle);
			// draw route
			if ((network.cities.Count > 1) && (network.lstEntities.Count > 0) && (network.iEntitySize > 0)) {
				Route r = network.FindBestRoute();
				int iTotal = (int) r.MeasureDistance(network.iDistances, network.iMaxTrip);
				int iHighlightStart = (int) (System.DateTime.Now.Ticks % iTotal);
				int iHighlightTotal = 40;  // total distance to highlight
				for (int i = 0; i < r.contents.Length + 2; i++) {
					City a = ((City) network.cities[(int) r.contents[i % r.contents.Length]]);
					City b = ((City) network.cities[(int) r.contents[(i + 1) % r.contents.Length]]);
					int iCurrentLeg = (int) Math.Sqrt(Math.Pow(a.x - b.x, 2) + Math.Pow(a.y - b.y, 2));
					g.DrawLine(Pens.Black, (int) a.x, (int) a.y, (int) b.x, (int) b.y);
					if (iCurrentLeg < iHighlightStart)
						iHighlightStart -= iCurrentLeg;
					else {  // must highlight at least part of this leg
						if (iCurrentLeg >= iHighlightStart + iHighlightTotal) {
						}
						else {
						}
					}
				}
			}
			// draw cities
			foreach (object cityobject in network.cities) {
				City city = (City) cityobject;
				g.FillEllipse(Brushes.Tan, (int) (city.x - 5), (int) (city.y - 5), 10, 10);
				g.DrawEllipse(Pens.Black, (int) (city.x - 5), (int) (city.y - 5), 10, 10);
				g.DrawString(city.name, font, Brushes.Black, (int) (city.x), (int) (city.y - 20), format);
			}
		}

		private void picMap_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e) {
			iMouseX = e.X;
			iMouseY = e.Y;
			if (movingCity != null) {
				movingCity.x = iMouseX;
				movingCity.y = iMouseY;
				network.SetDistances();
				picMap.Refresh();
			}
		}

		private void picMap_MouseLeave(object sender, System.EventArgs e) {
			movingCity = null;
		}

		private void picMap_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e) {
			City clickedCity = null;
			// check location first
			foreach (object cityobject in network.cities) {
				City city = (City) cityobject;
				if (Math.Abs(city.x - iMouseX) + Math.Abs(city.y - iMouseY) < 15) {  // close enough - delete city.
					clickedCity = city;
					break;
				}
			}
			if (e.Button == MouseButtons.Left) {  // deleting or placing city
				if (clickedCity != null)
					network.cities.Remove(clickedCity);
				else {  // placing new city
					int x = iMouseX;
					int y = iMouseY;  // preserve coordinates
					network.AddCity(x, y);
					network.Initialize();
				}
				network.SetDistances();
			}
			else if ((e.Button == MouseButtons.Right) && (clickedCity != null))  // start moving city
				movingCity = clickedCity;
			picMap.Refresh();
		}

		private void picMap_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e) {
			if ((e.Button == MouseButtons.Right) && (movingCity != null)) {
				movingCity = null;
				picMap.Refresh();
				network.SetDistances();
			}
		}

		private void timerMapUpdate_Elapsed(object sender, System.Timers.ElapsedEventArgs e) {
			picMap.Refresh();
		}

		private void timerRouteList_Elapsed(object sender, System.Timers.ElapsedEventArgs e) {
			SetRouteList();
		}

		private void TravelingSalesmanProblem_Closing(object sender, System.ComponentModel.CancelEventArgs e) {
			if (SolveThread != null) {
				SolveThread.Abort();
				SolveThread = null;
			}
		}

		#endregion

		#region XML data file save/load functions

		private void btnSave_Click(object sender, System.EventArgs e) {
			SaveFileDialog s = new SaveFileDialog();
			s.Filter = "Network Files (*.net)|*.net";
			s.OverwritePrompt = true;
			if (s.ShowDialog() == DialogResult.OK) {
				XmlTextWriter save = new XmlTextWriter(s.FileName, System.Text.Encoding.UTF8);
				save.Formatting = Formatting.Indented;
				save.IndentChar = ' ';
				save.WriteStartDocument();
				save.WriteStartElement("TravelingSalesmanSolution");
				save.Indentation++;
				// write parameters
				save.WriteStartElement("Parameters");
				save.Indentation++;
				save.WriteElementString("PopulationSize", network.iPopulationSize.ToString());
				save.WriteElementString("NumCities", network.iNumCities.ToString());
				save.WriteElementString("MutationRate", network.dMutationRate.ToString());
				save.WriteElementString("NewEntities", network.dNewEntities.ToString());
				save.WriteElementString("ReplacementRate", network.dReplacementRate.ToString());
				save.Indentation--;
				save.WriteEndElement();
				// write city data
				save.WriteStartElement("Cities");
				save.Indentation++;
				foreach (object cityobject in network.cities) {
					City city = (City) cityobject;
					save.WriteStartElement("City");
					save.Indentation++;
					save.WriteElementString("Name", city.name);
					save.WriteElementString("X", city.x.ToString());
					save.WriteElementString("Y", city.y.ToString());
					save.Indentation--;
					save.WriteEndElement();
				}
				save.Indentation--;
				save.WriteEndElement();
				// write route data
				save.WriteStartElement("Routes");
				save.Indentation++;
				foreach (object routeobject in network.lstEntities) {
					Route route = (Route) routeobject;
					save.WriteStartElement("Route");
					save.Indentation++;
					for (int i = 0; i < network.iEntitySize; i++)
						save.WriteElementString("Step", route.contents[i].ToString());
					save.Indentation--;
					save.WriteEndElement();
				}
				save.Indentation--;
				save.WriteEndElement();
				// close XML file
				save.Indentation--;
				save.WriteEndElement();
				save.WriteEndDocument();
				save.Close();
			}
		}

		private void btnLoad_Click(object sender, System.EventArgs e) {
			City city;
			TravelingSalesmanSolution newNetwork = new TravelingSalesmanSolution();
			newNetwork.cities = new ArrayList();
			OpenFileDialog open = new OpenFileDialog();
			open.Filter = "Network files (*.net)|*.net";
			if (open.ShowDialog() == DialogResult.OK) {  // selected an XML file
				try {
					XmlTextReader xtr = new XmlTextReader(open.FileName);
					XmlValidatingReader xvr = new XmlValidatingReader(xtr);
					xvr.ValidationType = ValidationType.Schema;
					xvr.ValidationEventHandler += new ValidationEventHandler(ValidationErrors);
					XmlSchemaCollection xsc = new XmlSchemaCollection();
					xsc.Add("", System.Reflection.Assembly.GetExecutingAssembly().Location.Substring(0, System.Reflection.Assembly.GetExecutingAssembly().Location.Length - 31) + @"Schema Files\Network.xsd");
					xvr.Schemas.Add(xsc);
					XmlDocument xd = new XmlDocument();
					XmlLoadErrors = "";
					xd.Load(xvr);
					if (XmlLoadErrors.Length > 0)
						MessageBox.Show("Errors loading network document. This does not appear to be a valid network file.", "Errors Loading Network Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
					else {
						foreach (XmlNode x1 in xd.ChildNodes) {
							if (x1.Name == "TravelingSalesmanSolution") {
								foreach (XmlNode x2 in x1.ChildNodes) {
									if (x2.Name == "Parameters") {
										foreach (XmlNode x3 in x2.ChildNodes) {
											if (x3.Name == "PopulationSize")
												newNetwork.iPopulationSize = Int32.Parse(x3.InnerText);
											else if (x3.Name == "NumCities")
												newNetwork.iNumCities = newNetwork.iEntitySize = Int32.Parse(x3.InnerText);
											else if (x3.Name == "MutationRate")
												newNetwork.dMutationRate = Double.Parse(x3.InnerText);
											else if (x3.Name == "NewEntities")
												newNetwork.dNewEntities = Double.Parse(x3.InnerText);
											else if (x3.Name == "ReplacementRate")
												newNetwork.dReplacementRate = Double.Parse(x3.InnerText);
										}
									}
									else if (x2.Name == "Cities") {
										foreach (XmlNode x3 in x2.ChildNodes) {
											if (x3.Name == "City") {
												city = new City(100, 100);  // stub
												foreach (XmlNode x4 in x3.ChildNodes) {
													if (x4.Name == "Name")
														city.name = x4.InnerText;
													else if (x4.Name == "X")
														city.x = Double.Parse(x4.InnerText);
													else if (x4.Name == "Y")
														city.y = Double.Parse(x4.InnerText);
												}
												newNetwork.cities.Add(city);
											}
										}
									}
									else if (x2.Name == "Routes") {
										foreach (XmlNode x3 in x2.ChildNodes) {
											if (x3.Name == "Route") {
												Route route = new Route(newNetwork.iEntitySize);
												int contentLength = 0;
												foreach (XmlNode x4 in x3.ChildNodes) {
             if ((x4.Name == "Step") && (contentLength < newNetwork.iEntitySize))
														route.contents[contentLength++] = Char.Parse(x4.InnerText);
												}
												newNetwork.lstEntities.Add(route);
											}
										}
									}
								}
							}
						}
						xtr.Close();
						if (XmlLoadErrors.Length == 0) {
							newNetwork.SetDistances();
							network = newNetwork;
							SetRouteList();
							picMap.Refresh();
							SetTextboxes();
						}
					}
				} catch (Exception ex) {
					MessageBox.Show(ex.Message);
				}
			}
		}
		public void ValidationErrors(object sender, ValidationEventArgs e) {
			XmlLoadErrors += e.Message + "\r\n";
		}

		#endregion

		#region Helper functions

		private void SetRouteList() {
			lstRoutes.BeginUpdate();
			lstRoutes.Columns.Clear();
			lstRoutes.Items.Clear();
			if (network.lstEntities.Count > 0) {
				// set up columns
				ColumnHeader ch = new ColumnHeader();
				ch.Width = 90;
				ch.Text = "Fitness";
				lstRoutes.Columns.Add(ch);
				ch = new ColumnHeader();
				ch.Width = 80;
				ch.Text = "Dist";
				lstRoutes.Columns.Add(ch);
				ch = new ColumnHeader();
				ch.Width = Math.Max(40, lstRoutes.Width - 180);
				ch.Text = "Route";
				lstRoutes.Columns.Add(ch);
				foreach (Route r in network.lstEntities) {
					string s = "";
					for (int i = 0; i < network.cities.Count; i++)
						s += (int) r.contents[i] + (i == network.cities.Count - 1 ? "" : ",");
					lstRoutes.Items.Add(new ListViewItem(new string[] { "" + (network.iMaxTrip - r.MeasureDistance(network.iDistances, network.iMaxTrip)), r.TotalDistance(network.iDistances).ToString(), s}));
				}
			}
			lstRoutes.EndUpdate();
			lstRoutes.Refresh();
		}

		private void SetTextboxes() {
			txtBuddingRate.Text = network.dBuddingRate.ToString();
			txtMutationRate.Text = network.dMutationRate.ToString();
			txtNewEntities.Text = network.dNewEntities.ToString();
			txtPopulationSize.Text = network.iPopulationSize.ToString();
			txtReplacementRate.Text = network.dReplacementRate.ToString();
			txtBuddingRate.Refresh();
			txtMutationRate.Refresh();
			txtNewEntities.Refresh();
			txtPopulationSize.Refresh();
			txtReplacementRate.Refresh();
		}

		private void SolveFull() {
			int iBestDistance = 999999;
			int iTrainingRounds = 1;
			int LastList = System.DateTime.Now.Minute * 60 + System.DateTime.Now.Second;
			Route rBestRoute;
			for (int i = 0; i < 150000; i++, iTrainingRounds++) {
				network.ReproducePopulation();
				int CurrentTick = System.DateTime.Now.Minute * 60 + System.DateTime.Now.Second;
				if (CurrentTick != LastList) {  // update status
					double dAvgFitness = 0.0;
					for (int j = 0; j < network.lstEntities.Count; j++)
						dAvgFitness += (double) ((Route) network.lstEntities[j]).dFitness;
					dAvgFitness /= (double) network.lstEntities.Count;
					rBestRoute = network.FindBestRoute();
					if (rBestRoute.TotalDistance(network.iDistances) < iBestDistance) {
						iBestDistance = rBestRoute.TotalDistance(network.iDistances);
						i = 0;
					}
					txtStatus.Text = "Status: Best route = " + rBestRoute.TotalDistance(network.iDistances) + ". Average fitness = " + dAvgFitness.ToString("#.###") + ". Training rounds = " + iTrainingRounds + ".";
					txtStatus.Refresh();
					LastList = CurrentTick;
				}
			}
			rBestRoute = network.FindBestRoute();
			txtStatus.Text = "Status: Solved. Best route = " + rBestRoute.TotalDistance(network.iDistances) + ". Training rounds = " + iTrainingRounds + ".";
			txtStatus.Refresh();
			btnSolveFull.Text = "Solve (Full)";
			btnSolveFull.Refresh();
			SolveThread = null;
		}

		#endregion

	}
}
